# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grafeas/grafeas_v1/proto/vulnerability.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from grafeas.grafeas_v1.proto import (
    common_pb2 as grafeas_dot_grafeas__v1_dot_proto_dot_common__pb2,
)
from grafeas.grafeas_v1.proto import (
    cvss_pb2 as grafeas_dot_grafeas__v1_dot_proto_dot_cvss__pb2,
)
from grafeas.grafeas_v1.proto import (
    package_pb2 as grafeas_dot_grafeas__v1_dot_proto_dot_package__pb2,
)


DESCRIPTOR = _descriptor.FileDescriptor(
    name="grafeas/grafeas_v1/proto/vulnerability.proto",
    package="grafeas.v1",
    syntax="proto3",
    serialized_options=b"\n\rio.grafeas.v1P\001ZFgoogle.golang.org/genproto/googleapis/grafeas/grafeas_v1/proto;grafeas\242\002\003GRA",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n,grafeas/grafeas_v1/proto/vulnerability.proto\x12\ngrafeas.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a%grafeas/grafeas_v1/proto/common.proto\x1a#grafeas/grafeas_v1/proto/cvss.proto\x1a&grafeas/grafeas_v1/proto/package.proto"\xfa\x06\n\x11VulnerabilityNote\x12\x12\n\ncvss_score\x18\x01 \x01(\x02\x12&\n\x08severity\x18\x02 \x01(\x0e\x32\x14.grafeas.v1.Severity\x12\x35\n\x07\x64\x65tails\x18\x03 \x03(\x0b\x32$.grafeas.v1.VulnerabilityNote.Detail\x12#\n\x07\x63vss_v3\x18\x04 \x01(\x0b\x32\x12.grafeas.v1.CVSSv3\x12\x44\n\x0fwindows_details\x18\x05 \x03(\x0b\x32+.grafeas.v1.VulnerabilityNote.WindowsDetail\x12\x36\n\x12source_update_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a\x8d\x03\n\x06\x44\x65tail\x12\x15\n\rseverity_name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x14\n\x0cpackage_type\x18\x03 \x01(\t\x12\x18\n\x10\x61\x66\x66\x65\x63ted_cpe_uri\x18\x04 \x01(\t\x12\x18\n\x10\x61\x66\x66\x65\x63ted_package\x18\x05 \x01(\t\x12\x33\n\x16\x61\x66\x66\x65\x63ted_version_start\x18\x06 \x01(\x0b\x32\x13.grafeas.v1.Version\x12\x31\n\x14\x61\x66\x66\x65\x63ted_version_end\x18\x07 \x01(\x0b\x32\x13.grafeas.v1.Version\x12\x15\n\rfixed_cpe_uri\x18\x08 \x01(\t\x12\x15\n\rfixed_package\x18\t \x01(\t\x12*\n\rfixed_version\x18\n \x01(\x0b\x32\x13.grafeas.v1.Version\x12\x13\n\x0bis_obsolete\x18\x0b \x01(\x08\x12\x36\n\x12source_update_time\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a\xbe\x01\n\rWindowsDetail\x12\x0f\n\x07\x63pe_uri\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12M\n\nfixing_kbs\x18\x04 \x03(\x0b\x32\x39.grafeas.v1.VulnerabilityNote.WindowsDetail.KnowledgeBase\x1a*\n\rKnowledgeBase\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0b\n\x03url\x18\x02 \x01(\t"\xbd\x04\n\x17VulnerabilityOccurrence\x12\x0c\n\x04type\x18\x01 \x01(\t\x12&\n\x08severity\x18\x02 \x01(\x0e\x32\x14.grafeas.v1.Severity\x12\x12\n\ncvss_score\x18\x03 \x01(\x02\x12G\n\rpackage_issue\x18\x04 \x03(\x0b\x32\x30.grafeas.v1.VulnerabilityOccurrence.PackageIssue\x12\x19\n\x11short_description\x18\x05 \x01(\t\x12\x18\n\x10long_description\x18\x06 \x01(\t\x12,\n\x0crelated_urls\x18\x07 \x03(\x0b\x32\x16.grafeas.v1.RelatedUrl\x12\x30\n\x12\x65\x66\x66\x65\x63tive_severity\x18\x08 \x01(\x0e\x32\x14.grafeas.v1.Severity\x12\x15\n\rfix_available\x18\t \x01(\x08\x1a\xe2\x01\n\x0cPackageIssue\x12\x18\n\x10\x61\x66\x66\x65\x63ted_cpe_uri\x18\x01 \x01(\t\x12\x18\n\x10\x61\x66\x66\x65\x63ted_package\x18\x02 \x01(\t\x12-\n\x10\x61\x66\x66\x65\x63ted_version\x18\x03 \x01(\x0b\x32\x13.grafeas.v1.Version\x12\x15\n\rfixed_cpe_uri\x18\x04 \x01(\t\x12\x15\n\rfixed_package\x18\x05 \x01(\t\x12*\n\rfixed_version\x18\x06 \x01(\x0b\x32\x13.grafeas.v1.Version\x12\x15\n\rfix_available\x18\x07 \x01(\x08*^\n\x08Severity\x12\x18\n\x14SEVERITY_UNSPECIFIED\x10\x00\x12\x0b\n\x07MINIMAL\x10\x01\x12\x07\n\x03LOW\x10\x02\x12\n\n\x06MEDIUM\x10\x03\x12\x08\n\x04HIGH\x10\x04\x12\x0c\n\x08\x43RITICAL\x10\x05\x42_\n\rio.grafeas.v1P\x01ZFgoogle.golang.org/genproto/googleapis/grafeas/grafeas_v1/proto;grafeas\xa2\x02\x03GRAb\x06proto3',
    dependencies=[
        google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,
        grafeas_dot_grafeas__v1_dot_proto_dot_common__pb2.DESCRIPTOR,
        grafeas_dot_grafeas__v1_dot_proto_dot_cvss__pb2.DESCRIPTOR,
        grafeas_dot_grafeas__v1_dot_proto_dot_package__pb2.DESCRIPTOR,
    ],
)

_SEVERITY = _descriptor.EnumDescriptor(
    name="Severity",
    full_name="grafeas.v1.Severity",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="SEVERITY_UNSPECIFIED",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="MINIMAL",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="LOW",
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="MEDIUM",
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="HIGH",
            index=4,
            number=4,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="CRITICAL",
            index=5,
            number=5,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=1678,
    serialized_end=1772,
)
_sym_db.RegisterEnumDescriptor(_SEVERITY)

Severity = enum_type_wrapper.EnumTypeWrapper(_SEVERITY)
SEVERITY_UNSPECIFIED = 0
MINIMAL = 1
LOW = 2
MEDIUM = 3
HIGH = 4
CRITICAL = 5


_VULNERABILITYNOTE_DETAIL = _descriptor.Descriptor(
    name="Detail",
    full_name="grafeas.v1.VulnerabilityNote.Detail",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="severity_name",
            full_name="grafeas.v1.VulnerabilityNote.Detail.severity_name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="description",
            full_name="grafeas.v1.VulnerabilityNote.Detail.description",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="package_type",
            full_name="grafeas.v1.VulnerabilityNote.Detail.package_type",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="affected_cpe_uri",
            full_name="grafeas.v1.VulnerabilityNote.Detail.affected_cpe_uri",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="affected_package",
            full_name="grafeas.v1.VulnerabilityNote.Detail.affected_package",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="affected_version_start",
            full_name="grafeas.v1.VulnerabilityNote.Detail.affected_version_start",
            index=5,
            number=6,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="affected_version_end",
            full_name="grafeas.v1.VulnerabilityNote.Detail.affected_version_end",
            index=6,
            number=7,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="fixed_cpe_uri",
            full_name="grafeas.v1.VulnerabilityNote.Detail.fixed_cpe_uri",
            index=7,
            number=8,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="fixed_package",
            full_name="grafeas.v1.VulnerabilityNote.Detail.fixed_package",
            index=8,
            number=9,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="fixed_version",
            full_name="grafeas.v1.VulnerabilityNote.Detail.fixed_version",
            index=9,
            number=10,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="is_obsolete",
            full_name="grafeas.v1.VulnerabilityNote.Detail.is_obsolete",
            index=10,
            number=11,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="source_update_time",
            full_name="grafeas.v1.VulnerabilityNote.Detail.source_update_time",
            index=11,
            number=12,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=510,
    serialized_end=907,
)

_VULNERABILITYNOTE_WINDOWSDETAIL_KNOWLEDGEBASE = _descriptor.Descriptor(
    name="KnowledgeBase",
    full_name="grafeas.v1.VulnerabilityNote.WindowsDetail.KnowledgeBase",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="name",
            full_name="grafeas.v1.VulnerabilityNote.WindowsDetail.KnowledgeBase.name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="url",
            full_name="grafeas.v1.VulnerabilityNote.WindowsDetail.KnowledgeBase.url",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1058,
    serialized_end=1100,
)

_VULNERABILITYNOTE_WINDOWSDETAIL = _descriptor.Descriptor(
    name="WindowsDetail",
    full_name="grafeas.v1.VulnerabilityNote.WindowsDetail",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="cpe_uri",
            full_name="grafeas.v1.VulnerabilityNote.WindowsDetail.cpe_uri",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="name",
            full_name="grafeas.v1.VulnerabilityNote.WindowsDetail.name",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="description",
            full_name="grafeas.v1.VulnerabilityNote.WindowsDetail.description",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="fixing_kbs",
            full_name="grafeas.v1.VulnerabilityNote.WindowsDetail.fixing_kbs",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[_VULNERABILITYNOTE_WINDOWSDETAIL_KNOWLEDGEBASE,],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=910,
    serialized_end=1100,
)

_VULNERABILITYNOTE = _descriptor.Descriptor(
    name="VulnerabilityNote",
    full_name="grafeas.v1.VulnerabilityNote",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="cvss_score",
            full_name="grafeas.v1.VulnerabilityNote.cvss_score",
            index=0,
            number=1,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="severity",
            full_name="grafeas.v1.VulnerabilityNote.severity",
            index=1,
            number=2,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="details",
            full_name="grafeas.v1.VulnerabilityNote.details",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="cvss_v3",
            full_name="grafeas.v1.VulnerabilityNote.cvss_v3",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="windows_details",
            full_name="grafeas.v1.VulnerabilityNote.windows_details",
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="source_update_time",
            full_name="grafeas.v1.VulnerabilityNote.source_update_time",
            index=5,
            number=6,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[_VULNERABILITYNOTE_DETAIL, _VULNERABILITYNOTE_WINDOWSDETAIL,],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=210,
    serialized_end=1100,
)


_VULNERABILITYOCCURRENCE_PACKAGEISSUE = _descriptor.Descriptor(
    name="PackageIssue",
    full_name="grafeas.v1.VulnerabilityOccurrence.PackageIssue",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="affected_cpe_uri",
            full_name="grafeas.v1.VulnerabilityOccurrence.PackageIssue.affected_cpe_uri",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="affected_package",
            full_name="grafeas.v1.VulnerabilityOccurrence.PackageIssue.affected_package",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="affected_version",
            full_name="grafeas.v1.VulnerabilityOccurrence.PackageIssue.affected_version",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="fixed_cpe_uri",
            full_name="grafeas.v1.VulnerabilityOccurrence.PackageIssue.fixed_cpe_uri",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="fixed_package",
            full_name="grafeas.v1.VulnerabilityOccurrence.PackageIssue.fixed_package",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="fixed_version",
            full_name="grafeas.v1.VulnerabilityOccurrence.PackageIssue.fixed_version",
            index=5,
            number=6,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="fix_available",
            full_name="grafeas.v1.VulnerabilityOccurrence.PackageIssue.fix_available",
            index=6,
            number=7,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1450,
    serialized_end=1676,
)

_VULNERABILITYOCCURRENCE = _descriptor.Descriptor(
    name="VulnerabilityOccurrence",
    full_name="grafeas.v1.VulnerabilityOccurrence",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="type",
            full_name="grafeas.v1.VulnerabilityOccurrence.type",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="severity",
            full_name="grafeas.v1.VulnerabilityOccurrence.severity",
            index=1,
            number=2,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="cvss_score",
            full_name="grafeas.v1.VulnerabilityOccurrence.cvss_score",
            index=2,
            number=3,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="package_issue",
            full_name="grafeas.v1.VulnerabilityOccurrence.package_issue",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="short_description",
            full_name="grafeas.v1.VulnerabilityOccurrence.short_description",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="long_description",
            full_name="grafeas.v1.VulnerabilityOccurrence.long_description",
            index=5,
            number=6,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="related_urls",
            full_name="grafeas.v1.VulnerabilityOccurrence.related_urls",
            index=6,
            number=7,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="effective_severity",
            full_name="grafeas.v1.VulnerabilityOccurrence.effective_severity",
            index=7,
            number=8,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="fix_available",
            full_name="grafeas.v1.VulnerabilityOccurrence.fix_available",
            index=8,
            number=9,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[_VULNERABILITYOCCURRENCE_PACKAGEISSUE,],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1103,
    serialized_end=1676,
)

_VULNERABILITYNOTE_DETAIL.fields_by_name[
    "affected_version_start"
].message_type = grafeas_dot_grafeas__v1_dot_proto_dot_package__pb2._VERSION
_VULNERABILITYNOTE_DETAIL.fields_by_name[
    "affected_version_end"
].message_type = grafeas_dot_grafeas__v1_dot_proto_dot_package__pb2._VERSION
_VULNERABILITYNOTE_DETAIL.fields_by_name[
    "fixed_version"
].message_type = grafeas_dot_grafeas__v1_dot_proto_dot_package__pb2._VERSION
_VULNERABILITYNOTE_DETAIL.fields_by_name[
    "source_update_time"
].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_VULNERABILITYNOTE_DETAIL.containing_type = _VULNERABILITYNOTE
_VULNERABILITYNOTE_WINDOWSDETAIL_KNOWLEDGEBASE.containing_type = (
    _VULNERABILITYNOTE_WINDOWSDETAIL
)
_VULNERABILITYNOTE_WINDOWSDETAIL.fields_by_name[
    "fixing_kbs"
].message_type = _VULNERABILITYNOTE_WINDOWSDETAIL_KNOWLEDGEBASE
_VULNERABILITYNOTE_WINDOWSDETAIL.containing_type = _VULNERABILITYNOTE
_VULNERABILITYNOTE.fields_by_name["severity"].enum_type = _SEVERITY
_VULNERABILITYNOTE.fields_by_name["details"].message_type = _VULNERABILITYNOTE_DETAIL
_VULNERABILITYNOTE.fields_by_name[
    "cvss_v3"
].message_type = grafeas_dot_grafeas__v1_dot_proto_dot_cvss__pb2._CVSSV3
_VULNERABILITYNOTE.fields_by_name[
    "windows_details"
].message_type = _VULNERABILITYNOTE_WINDOWSDETAIL
_VULNERABILITYNOTE.fields_by_name[
    "source_update_time"
].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_VULNERABILITYOCCURRENCE_PACKAGEISSUE.fields_by_name[
    "affected_version"
].message_type = grafeas_dot_grafeas__v1_dot_proto_dot_package__pb2._VERSION
_VULNERABILITYOCCURRENCE_PACKAGEISSUE.fields_by_name[
    "fixed_version"
].message_type = grafeas_dot_grafeas__v1_dot_proto_dot_package__pb2._VERSION
_VULNERABILITYOCCURRENCE_PACKAGEISSUE.containing_type = _VULNERABILITYOCCURRENCE
_VULNERABILITYOCCURRENCE.fields_by_name["severity"].enum_type = _SEVERITY
_VULNERABILITYOCCURRENCE.fields_by_name[
    "package_issue"
].message_type = _VULNERABILITYOCCURRENCE_PACKAGEISSUE
_VULNERABILITYOCCURRENCE.fields_by_name[
    "related_urls"
].message_type = grafeas_dot_grafeas__v1_dot_proto_dot_common__pb2._RELATEDURL
_VULNERABILITYOCCURRENCE.fields_by_name["effective_severity"].enum_type = _SEVERITY
DESCRIPTOR.message_types_by_name["VulnerabilityNote"] = _VULNERABILITYNOTE
DESCRIPTOR.message_types_by_name["VulnerabilityOccurrence"] = _VULNERABILITYOCCURRENCE
DESCRIPTOR.enum_types_by_name["Severity"] = _SEVERITY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

VulnerabilityNote = _reflection.GeneratedProtocolMessageType(
    "VulnerabilityNote",
    (_message.Message,),
    {
        "Detail": _reflection.GeneratedProtocolMessageType(
            "Detail",
            (_message.Message,),
            {
                "DESCRIPTOR": _VULNERABILITYNOTE_DETAIL,
                "__module__": "grafeas.grafeas_v1.proto.vulnerability_pb2",
                "__doc__": """A detail for a distro and package affected by this vulnerability and
    its associated fix (if one is available).
    
    Attributes:
        severity_name:
            The distro assigned severity of this vulnerability.
        description:
            A vendor-specific description of this vulnerability.
        package_type:
            The type of package; whether native or non native (e.g., ruby
            gems, node.js packages, etc.).
        affected_cpe_uri:
            Required. The `CPE URI
            <https://cpe.mitre.org/specification/>`__ this vulnerability
            affects.
        affected_package:
            Required. The package this vulnerability affects.
        affected_version_start:
            The version number at the start of an interval in which this
            vulnerability exists. A vulnerability can affect a package
            between version numbers that are disjoint sets of intervals
            (example: [1.0.0-1.1.0], [2.4.6-2.4.8] and [4.5.6-4.6.8]) each
            of which will be represented in its own Detail. If a specific
            affected version is provided by a vulnerability database,
            affected_version_start and affected_version_end will be the
            same in that Detail.
        affected_version_end:
            The version number at the end of an interval in which this
            vulnerability exists. A vulnerability can affect a package
            between version numbers that are disjoint sets of intervals
            (example: [1.0.0-1.1.0], [2.4.6-2.4.8] and [4.5.6-4.6.8]) each
            of which will be represented in its own Detail. If a specific
            affected version is provided by a vulnerability database,
            affected_version_start and affected_version_end will be the
            same in that Detail.
        fixed_cpe_uri:
            The distro recommended `CPE URI
            <https://cpe.mitre.org/specification/>`__ to update to that
            contains a fix for this vulnerability. It is possible for this
            to be different from the affected_cpe_uri.
        fixed_package:
            The distro recommended package to update to that contains a
            fix for this vulnerability. It is possible for this to be
            different from the affected_package.
        fixed_version:
            The distro recommended version to update to that contains a
            fix for this vulnerability. Setting this to
            VersionKind.MAXIMUM means no such version is yet available.
        is_obsolete:
            Whether this detail is obsolete. Occurrences are expected not
            to point to obsolete details.
        source_update_time:
            The time this information was last changed at the source. This
            is an upstream timestamp from the underlying information
            source - e.g. Ubuntu security tracker.
    """,
                # @@protoc_insertion_point(class_scope:grafeas.v1.VulnerabilityNote.Detail)
            },
        ),
        "WindowsDetail": _reflection.GeneratedProtocolMessageType(
            "WindowsDetail",
            (_message.Message,),
            {
                "KnowledgeBase": _reflection.GeneratedProtocolMessageType(
                    "KnowledgeBase",
                    (_message.Message,),
                    {
                        "DESCRIPTOR": _VULNERABILITYNOTE_WINDOWSDETAIL_KNOWLEDGEBASE,
                        "__module__": "grafeas.grafeas_v1.proto.vulnerability_pb2",
                        "__doc__": """
    Attributes:
          name:
              The KB name (generally of the form KB[0-9]+ (e.g., KB123456)).
          url:
              A link to the KB in the [Windows update catalog]
              (https://www.catalog.update.microsoft.com/).
      """,
                        # @@protoc_insertion_point(class_scope:grafeas.v1.VulnerabilityNote.WindowsDetail.KnowledgeBase)
                    },
                ),
                "DESCRIPTOR": _VULNERABILITYNOTE_WINDOWSDETAIL,
                "__module__": "grafeas.grafeas_v1.proto.vulnerability_pb2",
                "__doc__": """
    Attributes:
        cpe_uri:
            Required. The `CPE URI
            <https://cpe.mitre.org/specification/>`__ this vulnerability
            affects.
        name:
            Required. The name of this vulnerability.
        description:
            The description of this vulnerability.
        fixing_kbs:
            Required. The names of the KBs which have hotfixes to mitigate
            this vulnerability. Note that there may be multiple hotfixes
            (and thus multiple KBs) that mitigate a given vulnerability.
            Currently any listed KBs presence is considered a fix.
    """,
                # @@protoc_insertion_point(class_scope:grafeas.v1.VulnerabilityNote.WindowsDetail)
            },
        ),
        "DESCRIPTOR": _VULNERABILITYNOTE,
        "__module__": "grafeas.grafeas_v1.proto.vulnerability_pb2",
        "__doc__": """A security vulnerability that can be found in resources.
  
  Attributes:
      cvss_score:
          The CVSS score of this vulnerability. CVSS score is on a scale
          of 0 - 10 where 0 indicates low severity and 10 indicates high
          severity.
      severity:
          The note provider assigned severity of this vulnerability.
      details:
          Details of all known distros and packages affected by this
          vulnerability.
      cvss_v3:
          The full description of the CVSSv3 for this vulnerability.
      windows_details:
          Windows details get their own format because the information
          format and model don’t match a normal detail. Specifically
          Windows updates are done as patches, thus Windows
          vulnerabilities really are a missing package, rather than a
          package being at an incorrect version.
      source_update_time:
          The time this information was last changed at the source. This
          is an upstream timestamp from the underlying information
          source - e.g. Ubuntu security tracker.
  """,
        # @@protoc_insertion_point(class_scope:grafeas.v1.VulnerabilityNote)
    },
)
_sym_db.RegisterMessage(VulnerabilityNote)
_sym_db.RegisterMessage(VulnerabilityNote.Detail)
_sym_db.RegisterMessage(VulnerabilityNote.WindowsDetail)
_sym_db.RegisterMessage(VulnerabilityNote.WindowsDetail.KnowledgeBase)

VulnerabilityOccurrence = _reflection.GeneratedProtocolMessageType(
    "VulnerabilityOccurrence",
    (_message.Message,),
    {
        "PackageIssue": _reflection.GeneratedProtocolMessageType(
            "PackageIssue",
            (_message.Message,),
            {
                "DESCRIPTOR": _VULNERABILITYOCCURRENCE_PACKAGEISSUE,
                "__module__": "grafeas.grafeas_v1.proto.vulnerability_pb2",
                "__doc__": """A detail for a distro and package this vulnerability occurrence was
    found in and its associated fix (if one is available).
    
    Attributes:
        affected_cpe_uri:
            Required. The `CPE URI
            <https://cpe.mitre.org/specification/>`__ this vulnerability
            was found in.
        affected_package:
            Required. The package this vulnerability was found in.
        affected_version:
            Required. The version of the package that is installed on the
            resource affected by this vulnerability.
        fixed_cpe_uri:
            The `CPE URI <https://cpe.mitre.org/specification/>`__ this
            vulnerability was fixed in. It is possible for this to be
            different from the affected_cpe_uri.
        fixed_package:
            The package this vulnerability was fixed in. It is possible
            for this to be different from the affected_package.
        fixed_version:
            Required. The version of the package this vulnerability was
            fixed in. Setting this to VersionKind.MAXIMUM means no fix is
            yet available.
        fix_available:
            Output only. Whether a fix is available for this package.
    """,
                # @@protoc_insertion_point(class_scope:grafeas.v1.VulnerabilityOccurrence.PackageIssue)
            },
        ),
        "DESCRIPTOR": _VULNERABILITYOCCURRENCE,
        "__module__": "grafeas.grafeas_v1.proto.vulnerability_pb2",
        "__doc__": """An occurrence of a severity vulnerability on a resource.
  
  Attributes:
      type:
          The type of package; whether native or non native (e.g., ruby
          gems, node.js packages, etc.).
      severity:
          Output only. The note provider assigned severity of this
          vulnerability.
      cvss_score:
          Output only. The CVSS score of this vulnerability. CVSS score
          is on a scale of 0 - 10 where 0 indicates low severity and 10
          indicates high severity.
      package_issue:
          Required. The set of affected locations and their fixes (if
          available) within the associated resource.
      short_description:
          Output only. A one sentence description of this vulnerability.
      long_description:
          Output only. A detailed description of this vulnerability.
      related_urls:
          Output only. URLs related to this vulnerability.
      effective_severity:
          The distro assigned severity for this vulnerability when it is
          available, otherwise this is the note provider assigned
          severity.
      fix_available:
          Output only. Whether at least one of the affected packages has
          a fix available.
  """,
        # @@protoc_insertion_point(class_scope:grafeas.v1.VulnerabilityOccurrence)
    },
)
_sym_db.RegisterMessage(VulnerabilityOccurrence)
_sym_db.RegisterMessage(VulnerabilityOccurrence.PackageIssue)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
#
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unit tests."""

import mock
import pytest

from google.protobuf import empty_pb2
from grafeas import grafeas_v1
from grafeas.grafeas_v1.proto import grafeas_pb2
from grafeas.grafeas_v1.gapic.transports import grafeas_grpc_transport


class MultiCallableStub(object):
    """Stub for the grpc.UnaryUnaryMultiCallable interface."""

    def __init__(self, method, channel_stub):
        self.method = method
        self.channel_stub = channel_stub

    def __call__(self, request, timeout=None, metadata=None, credentials=None):
        self.channel_stub.requests.append((self.method, request))

        response = None
        if self.channel_stub.responses:
            response = self.channel_stub.responses.pop()

        if isinstance(response, Exception):
            raise response

        if response:
            return response


class ChannelStub(object):
    """Stub for the grpc.Channel interface."""

    def __init__(self, responses=[]):
        self.responses = responses
        self.requests = []

    def unary_unary(self, method, request_serializer=None, response_deserializer=None):
        return MultiCallableStub(method, self)


class CustomException(Exception):
    pass


class TestGrafeasClient(object):
    def test_get_occurrence(self):
        # Setup Expected Response
        name_2 = "name2-1052831874"
        resource_uri = "resourceUri-384040517"
        note_name = "noteName1780787896"
        remediation = "remediation779381797"
        expected_response = {
            "name": name_2,
            "resource_uri": resource_uri,
            "note_name": note_name,
            "remediation": remediation,
        }
        expected_response = grafeas_pb2.Occurrence(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup Request
        name = client.occurrence_path("[PROJECT]", "[OCCURRENCE]")

        response = client.get_occurrence(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = grafeas_pb2.GetOccurrenceRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_get_occurrence_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup request
        name = client.occurrence_path("[PROJECT]", "[OCCURRENCE]")

        with pytest.raises(CustomException):
            client.get_occurrence(name)

    def test_list_occurrences(self):
        # Setup Expected Response
        next_page_token = ""
        occurrences_element = {}
        occurrences = [occurrences_element]
        expected_response = {
            "next_page_token": next_page_token,
            "occurrences": occurrences,
        }
        expected_response = grafeas_pb2.ListOccurrencesResponse(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup Request
        parent = client.project_path("[PROJECT]")

        paged_list_response = client.list_occurrences(parent)
        resources = list(paged_list_response)
        assert len(resources) == 1

        assert expected_response.occurrences[0] == resources[0]

        assert len(channel.requests) == 1
        expected_request = grafeas_pb2.ListOccurrencesRequest(parent=parent)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_occurrences_exception(self):
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup request
        parent = client.project_path("[PROJECT]")

        paged_list_response = client.list_occurrences(parent)
        with pytest.raises(CustomException):
            list(paged_list_response)

    def test_delete_occurrence(self):
        channel = ChannelStub()
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup Request
        name = client.occurrence_path("[PROJECT]", "[OCCURRENCE]")

        client.delete_occurrence(name)

        assert len(channel.requests) == 1
        expected_request = grafeas_pb2.DeleteOccurrenceRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_delete_occurrence_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup request
        name = client.occurrence_path("[PROJECT]", "[OCCURRENCE]")

        with pytest.raises(CustomException):
            client.delete_occurrence(name)

    def test_create_occurrence(self):
        # Setup Expected Response
        name = "name3373707"
        resource_uri = "resourceUri-384040517"
        note_name = "noteName1780787896"
        remediation = "remediation779381797"
        expected_response = {
            "name": name,
            "resource_uri": resource_uri,
            "note_name": note_name,
            "remediation": remediation,
        }
        expected_response = grafeas_pb2.Occurrence(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup Request
        parent = client.project_path("[PROJECT]")
        occurrence = {}

        response = client.create_occurrence(parent, occurrence)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = grafeas_pb2.CreateOccurrenceRequest(
            parent=parent, occurrence=occurrence
        )
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_create_occurrence_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup request
        parent = client.project_path("[PROJECT]")
        occurrence = {}

        with pytest.raises(CustomException):
            client.create_occurrence(parent, occurrence)

    def test_batch_create_occurrences(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = grafeas_pb2.BatchCreateOccurrencesResponse(
            **expected_response
        )

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup Request
        parent = client.project_path("[PROJECT]")
        occurrences = []

        response = client.batch_create_occurrences(parent, occurrences)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = grafeas_pb2.BatchCreateOccurrencesRequest(
            parent=parent, occurrences=occurrences
        )
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_batch_create_occurrences_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup request
        parent = client.project_path("[PROJECT]")
        occurrences = []

        with pytest.raises(CustomException):
            client.batch_create_occurrences(parent, occurrences)

    def test_update_occurrence(self):
        # Setup Expected Response
        name_2 = "name2-1052831874"
        resource_uri = "resourceUri-384040517"
        note_name = "noteName1780787896"
        remediation = "remediation779381797"
        expected_response = {
            "name": name_2,
            "resource_uri": resource_uri,
            "note_name": note_name,
            "remediation": remediation,
        }
        expected_response = grafeas_pb2.Occurrence(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup Request
        name = client.occurrence_path("[PROJECT]", "[OCCURRENCE]")
        occurrence = {}

        response = client.update_occurrence(name, occurrence)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = grafeas_pb2.UpdateOccurrenceRequest(
            name=name, occurrence=occurrence
        )
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_update_occurrence_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup request
        name = client.occurrence_path("[PROJECT]", "[OCCURRENCE]")
        occurrence = {}

        with pytest.raises(CustomException):
            client.update_occurrence(name, occurrence)

    def test_get_occurrence_note(self):
        # Setup Expected Response
        name_2 = "name2-1052831874"
        short_description = "shortDescription-235369287"
        long_description = "longDescription-1747792199"
        expected_response = {
            "name": name_2,
            "short_description": short_description,
            "long_description": long_description,
        }
        expected_response = grafeas_pb2.Note(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup Request
        name = client.occurrence_path("[PROJECT]", "[OCCURRENCE]")

        response = client.get_occurrence_note(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = grafeas_pb2.GetOccurrenceNoteRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_get_occurrence_note_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup request
        name = client.occurrence_path("[PROJECT]", "[OCCURRENCE]")

        with pytest.raises(CustomException):
            client.get_occurrence_note(name)

    def test_get_note(self):
        # Setup Expected Response
        name_2 = "name2-1052831874"
        short_description = "shortDescription-235369287"
        long_description = "longDescription-1747792199"
        expected_response = {
            "name": name_2,
            "short_description": short_description,
            "long_description": long_description,
        }
        expected_response = grafeas_pb2.Note(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup Request
        name = client.note_path("[PROJECT]", "[NOTE]")

        response = client.get_note(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = grafeas_pb2.GetNoteRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_get_note_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup request
        name = client.note_path("[PROJECT]", "[NOTE]")

        with pytest.raises(CustomException):
            client.get_note(name)

    def test_list_notes(self):
        # Setup Expected Response
        next_page_token = ""
        notes_element = {}
        notes = [notes_element]
        expected_response = {"next_page_token": next_page_token, "notes": notes}
        expected_response = grafeas_pb2.ListNotesResponse(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup Request
        parent = client.project_path("[PROJECT]")

        paged_list_response = client.list_notes(parent)
        resources = list(paged_list_response)
        assert len(resources) == 1

        assert expected_response.notes[0] == resources[0]

        assert len(channel.requests) == 1
        expected_request = grafeas_pb2.ListNotesRequest(parent=parent)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_notes_exception(self):
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup request
        parent = client.project_path("[PROJECT]")

        paged_list_response = client.list_notes(parent)
        with pytest.raises(CustomException):
            list(paged_list_response)

    def test_delete_note(self):
        channel = ChannelStub()
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup Request
        name = client.note_path("[PROJECT]", "[NOTE]")

        client.delete_note(name)

        assert len(channel.requests) == 1
        expected_request = grafeas_pb2.DeleteNoteRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_delete_note_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup request
        name = client.note_path("[PROJECT]", "[NOTE]")

        with pytest.raises(CustomException):
            client.delete_note(name)

    def test_create_note(self):
        # Setup Expected Response
        name = "name3373707"
        short_description = "shortDescription-235369287"
        long_description = "longDescription-1747792199"
        expected_response = {
            "name": name,
            "short_description": short_description,
            "long_description": long_description,
        }
        expected_response = grafeas_pb2.Note(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup Request
        parent = client.project_path("[PROJECT]")
        note_id = "noteId2129224840"
        note = {}

        response = client.create_note(parent, note_id, note)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = grafeas_pb2.CreateNoteRequest(
            parent=parent, note_id=note_id, note=note
        )
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_create_note_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup request
        parent = client.project_path("[PROJECT]")
        note_id = "noteId2129224840"
        note = {}

        with pytest.raises(CustomException):
            client.create_note(parent, note_id, note)

    def test_batch_create_notes(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = grafeas_pb2.BatchCreateNotesResponse(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup Request
        parent = client.project_path("[PROJECT]")
        notes = {}

        response = client.batch_create_notes(parent, notes)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = grafeas_pb2.BatchCreateNotesRequest(
            parent=parent, notes=notes
        )
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_batch_create_notes_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup request
        parent = client.project_path("[PROJECT]")
        notes = {}

        with pytest.raises(CustomException):
            client.batch_create_notes(parent, notes)

    def test_update_note(self):
        # Setup Expected Response
        name_2 = "name2-1052831874"
        short_description = "shortDescription-235369287"
        long_description = "longDescription-1747792199"
        expected_response = {
            "name": name_2,
            "short_description": short_description,
            "long_description": long_description,
        }
        expected_response = grafeas_pb2.Note(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup Request
        name = client.note_path("[PROJECT]", "[NOTE]")
        note = {}

        response = client.update_note(name, note)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = grafeas_pb2.UpdateNoteRequest(name=name, note=note)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_update_note_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup request
        name = client.note_path("[PROJECT]", "[NOTE]")
        note = {}

        with pytest.raises(CustomException):
            client.update_note(name, note)

    def test_list_note_occurrences(self):
        # Setup Expected Response
        next_page_token = ""
        occurrences_element = {}
        occurrences = [occurrences_element]
        expected_response = {
            "next_page_token": next_page_token,
            "occurrences": occurrences,
        }
        expected_response = grafeas_pb2.ListNoteOccurrencesResponse(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup Request
        name = client.note_path("[PROJECT]", "[NOTE]")

        paged_list_response = client.list_note_occurrences(name)
        resources = list(paged_list_response)
        assert len(resources) == 1

        assert expected_response.occurrences[0] == resources[0]

        assert len(channel.requests) == 1
        expected_request = grafeas_pb2.ListNoteOccurrencesRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_note_occurrences_exception(self):
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            address = "[SERVICE_ADDRESS]"

            scopes = "SCOPE"

            transport = grafeas_grpc_transport.GrafeasGrpcTransport(address, scopes)

            client = grafeas_v1.GrafeasClient(transport)

        # Setup request
        name = client.note_path("[PROJECT]", "[NOTE]")

        paged_list_response = client.list_note_occurrences(name)
        with pytest.raises(CustomException):
            list(paged_list_response)
